/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { getElement, getTestElement } from '../../support/cypress-helper';
import { ExpansionPanelE2Eomponent } from '../ui/expansion-panel.e2e.component';
import { VorgangFormularDatenHistorieE2EComponent } from './vorgang.formular-daten.historie.e2e.component';

export class VorgangFormularDatenE2EComponent {
  private readonly locatorRoot: string = 'formulardaten-panel';
  private readonly locatorMetadaten: string = 'metadaten';
  private readonly locatorAntragdaten: string = 'antragdaten';
  private readonly locatorTabset: string = 'div[role=tab]';

  private readonly vorgangFormPanelContainer: ExpansionPanelE2Eomponent =
    new ExpansionPanelE2Eomponent();
  private readonly historieContainer: VorgangFormularDatenHistorieE2EComponent =
    new VorgangFormularDatenHistorieE2EComponent();

  private readonly antragdatenTab: number = 0;
  private readonly metadatenTab: number = 1;
  private readonly representationTab: number = 2;
  private readonly attachmentTab: number = 3;
  private readonly historieTab: number = 4;

  public getRoot() {
    return cy.getTestElement(this.locatorRoot);
  }

  public getExpansionPanelContainer(): ExpansionPanelE2Eomponent {
    return this.vorgangFormPanelContainer;
  }

  public getAntragdatenTab() {
    return this.getTabsetElement().eq(this.antragdatenTab);
  }

  public getMetadatenTab() {
    return this.getTabsetElement().eq(this.metadatenTab);
  }

  public getRepresentationTab() {
    return this.getTabsetElement().eq(this.representationTab);
  }

  public getAttachmentTab() {
    return this.getTabsetElement().eq(this.attachmentTab);
  }

  public getHistorieTab() {
    return this.getTabsetElement().eq(this.historieTab);
  }

  private getTabsetElement() {
    return getElement(this.locatorTabset);
  }

  public getMetadaten() {
    return getTestElement(this.locatorMetadaten);
  }

  public getAntragdaten() {
    return getTestElement(this.locatorAntragdaten);
  }

  public getHistorieContainer(): VorgangFormularDatenHistorieE2EComponent {
    return this.historieContainer;
  }
}
