import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { NavigationE2EComponent } from 'apps/alfa-e2e/src/components/navigation/navigation.e2e.component';
import { VorgangListItemE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { VorgangSearchE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-search.e2e.component';
import {
  VorgangViewE2EComponent,
  VorgangViewsE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang-views.e2e.component';
import {
  ClientAttributeNameE2E,
  VorgangE2E,
  VorgangStatusE2E,
} from 'apps/alfa-e2e/src/model/vorgang';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import {
  isButtonToggleChecked,
  isButtonToggleNotChecked,
} from 'apps/alfa-e2e/src/support/angular.util';
import { dropCollections, dropSearchIndex } from 'apps/alfa-e2e/src/support/cypress-helper';
import {
  containClass,
  contains,
  enterWith,
  exist,
  haveLength,
  haveText,
  notContainClass,
} from 'apps/alfa-e2e/src/support/cypress.util';
import { createDateToday } from 'apps/alfa-e2e/src/support/tech.util';
import {
  getUserSabineId,
  initUsermanagerUsers,
  loginAsSabine,
} from 'apps/alfa-e2e/src/support/user-util';
import {
  buildVorgang,
  createAlfaClientAttributes,
  createNextWiedervorlageFristClientAttribute,
  createVorgang,
  initSearchIndex,
  initVorgaenge,
  objectIds,
} from 'apps/alfa-e2e/src/support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang views and filter', () => {
  const mainPage: MainPage = new MainPage();

  const header: HeaderE2EComponent = mainPage.getHeader();
  const vorgangSearch: VorgangSearchE2EComponent = header.getVorgangSearch();

  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const views: VorgangViewsE2EComponent = mainPage.getViews();
  const neuView: VorgangViewE2EComponent = views.getNeu();
  const angenommenView: VorgangViewE2EComponent = views.getAngenommen();
  const inBearbeitungView: VorgangViewE2EComponent = views.getInBearbeitung();
  const beschiedenView: VorgangViewE2EComponent = views.getBeschieden();
  const abgeschlossenView: VorgangViewE2EComponent = views.getAbgeschlossen();
  const verworfenView: VorgangViewE2EComponent = views.getVerworfen();
  const zuLoeschenView: VorgangViewE2EComponent = views.getZuLoeschen();
  const wiedervorlagenView: VorgangViewE2EComponent = views.getWiedervorlagen();
  const vorgangListView: VorgangViewE2EComponent = views.getVorgangList();
  const sucheView: VorgangViewE2EComponent = views.getSuche();

  const navigation: NavigationE2EComponent = mainPage.getNavigation();

  const vorgangNeuAssigned: VorgangE2E = {
    ...createVorgang(),
    name: 'VorgangNeuAssigned',
    status: VorgangStatusE2E.NEU,
    assignedTo: getUserSabineId(),
  };
  const vorgangNeuUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'VorgangNeuUnassigned'),
    status: VorgangStatusE2E.NEU,
  };

  const vorgangAngenommenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'VorgangAngenommenAssigned'),
    status: VorgangStatusE2E.ANGENOMMEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangAngenommenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'VorgangAngenommenUnassigned'),
    status: VorgangStatusE2E.ANGENOMMEN,
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateToday()),
    ),
  };

  const vorgangInBearbeitungAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[3], 'VorgangInBearbeitungAssigned'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
    assignedTo: getUserSabineId(),
  };
  const vorgangInBearbeitungUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[4], 'VorgangInBearbeitungUnassigned'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangBeschiedenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[5], 'VorgangBeschiedenAssigned'),
    status: VorgangStatusE2E.BESCHIEDEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangBeschiedenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[6], 'VorgangBeschiedenUnassigned'),
    status: VorgangStatusE2E.BESCHIEDEN,
  };

  const vorgangAbgeschlossenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[7], 'VorgangAbgeschlossenAssigned'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
    assignedTo: getUserSabineId(),
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateToday()),
    ),
  };

  const vorgangAbgeschlossenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[8], 'VorgangAbgeschlossenUnassigned'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
  };

  const vorgangVerworfenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[9], 'VorgangVerworfenAssigned'),
    status: VorgangStatusE2E.VERWORFEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangVerworfenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[10], 'VorgangVerworfenUnassigned'),
    status: VorgangStatusE2E.VERWORFEN,
  };

  const vorgangZuLoeschenAssigned: VorgangE2E = {
    ...buildVorgang(objectIds[11], 'VorgangZuLoeschenAssigned'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
    assignedTo: getUserSabineId(),
  };
  const vorgangZuLoeschenUnassigned: VorgangE2E = {
    ...buildVorgang(objectIds[12], 'VorgangZuLoeschenUnassigned'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
  };

  const vorgangNeuAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangNeuAssigned.name,
  );
  const vorgangNeuUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangNeuUnassigned.name,
  );

  const vorgangAngenommenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAngenommenAssigned.name,
  );
  const vorgangAngenommenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAngenommenUnassigned.name,
  );

  const vorgangInBearbeitungAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangInBearbeitungAssigned.name,
  );
  const vorgangInBearbeitungUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangInBearbeitungUnassigned.name,
  );

  const vorgangBeschiedenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangBeschiedenAssigned.name,
  );
  const vorgangBeschiedenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangBeschiedenUnassigned.name,
  );

  const vorgangAbgeschlossenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAbgeschlossenAssigned.name,
  );
  const vorgangAbgeschlossenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangAbgeschlossenUnassigned.name,
  );

  const vorgangVerworfenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangVerworfenAssigned.name,
  );
  const vorgangVerworfenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangVerworfenUnassigned.name,
  );

  const vorgangZuLoeschenAssignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangZuLoeschenAssigned.name,
  );
  const vorgangZuLoeschenUnassignedInList: VorgangListItemE2EComponent = vorgangList.getListItem(
    vorgangZuLoeschenUnassigned.name,
  );

  const LIST_SIZE: number = 14;

  before(() => {
    const vorgaenge: VorgangE2E[] = [
      vorgangNeuAssigned,
      vorgangNeuUnassigned,
      vorgangAngenommenAssigned,
      vorgangAngenommenUnassigned,
      vorgangInBearbeitungAssigned,
      vorgangInBearbeitungUnassigned,
      vorgangBeschiedenAssigned,
      vorgangBeschiedenUnassigned,
      vorgangAbgeschlossenAssigned,
      vorgangAbgeschlossenUnassigned,
      vorgangVerworfenAssigned,
      vorgangVerworfenUnassigned,
      vorgangZuLoeschenAssigned,
      vorgangZuLoeschenUnassigned,
    ];
    initVorgaenge(vorgaenge);

    initSearchIndex(vorgaenge);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
  });

  after(() => {
    dropCollections();
    dropSearchIndex();
  });

  describe('Initial', () => {
    describe('on load', () => {
      it('should show vorgaenge in vorgangList', () => {
        exist(vorgangList.getRoot());

        haveLength(vorgangList.getItems(), LIST_SIZE);
      });
    });

    describe('filter', () => {
      it('Alle should be selected', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
      });

      it('Meine Vorgänge should not be selected', () => {
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('Nicht zugewiesen should not be selected', () => {
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });
    });

    describe('view', () => {
      it('vorgangList should be selected', () => {
        isSelected(vorgangListView.getRoot());
      });

      it('other should be unselected', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      describe('count for', () => {
        it('neu', () => {
          contains(neuView.getCount(), '2');
        });

        it('angenommen', () => {
          contains(angenommenView.getCount(), '2');
        });

        it('in_bearbeitung', () => {
          contains(inBearbeitungView.getCount(), '2');
        });

        it('beschieden', () => {
          contains(beschiedenView.getCount(), '2');
        });

        it('abgeschlossen', () => {
          contains(abgeschlossenView.getCount(), '2');
        });

        it('verworfen', () => {
          contains(verworfenView.getCount(), '2');
        });

        it('zu löschen', () => {
          contains(zuLoeschenView.getCount(), '2');
        });

        it('wiedervorlagen', () => {
          contains(wiedervorlagenView.getCount(), '2');
        });
      });
    });
  });

  describe('Select view', () => {
    describe('Neu', () => {
      it('should set selected after click', () => {
        neuView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(neuView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), 2);
        exist(vorgangNeuAssignedInList.getRoot());
        exist(vorgangNeuUnassignedInList.getRoot());
      });
    });

    describe('Angenommen', () => {
      it('should set selected after click', () => {
        angenommenView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(angenommenView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAngenommenUnassignedInList.getRoot());
      });
    });

    describe('In Bearbeitung', () => {
      it('should set selected after click', () => {
        inBearbeitungView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(inBearbeitungView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE - 12);
        exist(vorgangInBearbeitungAssignedInList.getRoot());
        exist(vorgangInBearbeitungUnassignedInList.getRoot());
      });
    });

    describe('Beschieden', () => {
      it('should set selected after click', () => {
        beschiedenView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(beschiedenView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE - 12);
        exist(vorgangBeschiedenAssignedInList.getRoot());
        exist(vorgangBeschiedenUnassignedInList.getRoot());
      });
    });

    describe('Abgeschlossen', () => {
      it('should set selected after click', () => {
        abgeschlossenView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(abgeschlossenView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE - 12);
        exist(vorgangAbgeschlossenAssignedInList.getRoot());
        exist(vorgangAbgeschlossenUnassignedInList.getRoot());
      });
    });

    describe('Verworfen', () => {
      it('should set selected after click', () => {
        verworfenView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(verworfenView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE - 12);
        exist(vorgangVerworfenAssignedInList.getRoot());
        exist(vorgangVerworfenUnassignedInList.getRoot());
      });
    });

    describe('Zu Löschen', () => {
      it('should set selected after click', () => {
        zuLoeschenView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(zuLoeschenView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE - 12);
        exist(vorgangZuLoeschenAssignedInList.getRoot());
        exist(vorgangZuLoeschenUnassignedInList.getRoot());
      });
    });

    describe('Wiedervorlagen', () => {
      it('should set selected after click', () => {
        wiedervorlagenView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(wiedervorlagenView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should filter List', () => {
        haveLength(vorgangList.getItems(), 2);

        exist(vorgangAngenommenUnassignedInList.getRoot());
        exist(vorgangAbgeschlossenAssignedInList.getRoot());
      });
    });

    describe('VorgangList', () => {
      it('should set selected after click', () => {
        vorgangListView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(vorgangListView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(sucheView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should show full List', () => {
        haveLength(vorgangList.getItems(), LIST_SIZE);
      });
    });

    describe('Suche', () => {
      it('should set selected after click', () => {
        sucheView.getRoot().click();
        waitForSpinnerToDisappear();

        isSelected(sucheView.getRoot());
      });

      it('should unselect other views', () => {
        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should show message', () => {
        haveText(vorgangList.getEmptySucheMessage(), 'Bitte geben Sie einen Suchbegriff ein.');
      });
    });
  });

  describe('Select Filter', () => {
    describe('Nicht zugewiesen', () => {
      it('should set selected after click', () => {
        navigation.getNichtZugewiesenFilter().click();
        waitForSpinnerToDisappear();

        isButtonToggleChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should unselect Meine Vorgänge', () => {
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep Alle unselected', () => {
        isButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });

      it('should show message', () => {
        haveText(vorgangList.getEmptySucheMessage(), 'Bitte geben Sie einen Suchbegriff ein.');
      });
    });

    describe('Meine Vorgänge', () => {
      it('should set selected after click', () => {
        navigation.getMeineVorgaengeFilter().click();
        waitForSpinnerToDisappear();

        isButtonToggleChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should unselect Alle', () => {
        isButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });

      it('should show message', () => {
        haveText(vorgangList.getEmptySucheMessage(), 'Bitte geben Sie einen Suchbegriff ein.');
      });
    });
  });

  describe('Suche', () => {
    describe('on execute/enter with Meine Vorgänge filter selected', () => {
      it('should show filtered list', () => {
        doSearch('Angenommen');
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 1);
        exist(vorgangAngenommenAssignedInList.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getMeineVorgaengeFilter());
        isButtonToggleNotChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Alle filter selection', () => {
      it('should show filtered list', () => {
        navigation.getAlleFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAngenommenUnassignedInList.getRoot());
      });

      it('should select Alle filter', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
      });

      it('should unselect Meine Vorgänge filter', () => {
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Nicht zugewiesen filter selection', () => {
      it('should show filtered list', () => {
        navigation.getNichtZugewiesenFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 1);
        exist(vorgangAngenommenUnassignedInList.getRoot());
      });

      it('should show Nicht zugewiesen as selected', () => {
        isButtonToggleChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should unselect Alle filter', () => {
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep Meine Vorgänge unselected', () => {
        isButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on other search execution', () => {
      it('should show filtered list', () => {
        navigation.getAlleFilter().click();
        waitForSpinnerToDisappear();

        doSearch('VorgangA');
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 4);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAngenommenUnassignedInList.getRoot());
        exist(vorgangAbgeschlossenAssignedInList.getRoot());
        exist(vorgangAbgeschlossenUnassignedInList.getRoot());
      });

      it('should keep filter selection', () => {
        isButtonToggleChecked(navigation.getAlleFilter());
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should unselect Meine Vorgänge filter', () => {
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Meine Vorgänge filter selection', () => {
      it('should show filtered list', () => {
        navigation.getMeineVorgaengeFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenAssignedInList.getRoot());
        exist(vorgangAbgeschlossenAssignedInList.getRoot());
      });

      it('should select', () => {
        isButtonToggleChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should unselect Alle filter', () => {
        isButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });

    describe('on Nicht zugewiesen filter selection', () => {
      it('should show filtered list', () => {
        navigation.getNichtZugewiesenFilter().click();
        waitForSpinnerToDisappear();

        haveLength(vorgangList.getItems(), 2);
        exist(vorgangAngenommenUnassignedInList.getRoot());
        exist(vorgangAbgeschlossenUnassignedInList.getRoot());
      });

      it('should select', () => {
        isButtonToggleChecked(navigation.getNichtZugewiesenFilter());
      });

      it('should unselect Meine Vorgänge filter', () => {
        isButtonToggleNotChecked(navigation.getMeineVorgaengeFilter());
      });

      it('should keep filter unselected', () => {
        isButtonToggleNotChecked(navigation.getAlleFilter());
      });

      it('should keep view selection', () => {
        isSelected(sucheView.getRoot());

        isNotSelected(neuView.getRoot());
        isNotSelected(angenommenView.getRoot());
        isNotSelected(inBearbeitungView.getRoot());
        isNotSelected(beschiedenView.getRoot());
        isNotSelected(abgeschlossenView.getRoot());
        isNotSelected(verworfenView.getRoot());
        isNotSelected(zuLoeschenView.getRoot());
        isNotSelected(vorgangListView.getRoot());
      });
    });
  });

  function doSearch(searchString: string): void {
    enterWith(vorgangSearch.getInput(), searchString);
  }

  function isNotSelected(element) {
    notContainClass(element, 'selected');
  }

  function isSelected(element) {
    containClass(element, 'selected');
  }
});
