import {
  AbstractFormService,
  EMPTY_STRING,
  HttpError,
  ProblemDetail,
  StateResource,
} from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { FormControl, UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { isNil } from 'lodash-es';
import { Observable, map } from 'rxjs';
import { Postfach, PostfachResource } from '../../postfach.model';
import { PostfachService } from '../../postfach.service';

@Injectable()
export class PostfachFormService extends AbstractFormService<PostfachResource> {
  public static readonly ASBSENDER_GROUP: string = 'absender';
  public static readonly NAME_FIELD: string = 'name';
  public static readonly ANSCHRIFT_FIELD: string = 'anschrift';
  public static readonly DIENST_FIELD: string = 'dienst';
  public static readonly MANDANT_FIELD: string = 'mandant';
  public static readonly GEMEINDESCHLUESSEL_FIELD: string = 'gemeindeschluessel';

  public static readonly SIGNATUR_FIELD: string = 'signatur';

  constructor(
    formBuilder: UntypedFormBuilder,
    private postfachService: PostfachService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [PostfachFormService.ASBSENDER_GROUP]: this.formBuilder.group({
        [PostfachFormService.NAME_FIELD]: new FormControl(EMPTY_STRING),
        [PostfachFormService.ANSCHRIFT_FIELD]: new FormControl(EMPTY_STRING),
        [PostfachFormService.DIENST_FIELD]: new FormControl(EMPTY_STRING),
        [PostfachFormService.MANDANT_FIELD]: new FormControl(EMPTY_STRING),
        [PostfachFormService.GEMEINDESCHLUESSEL_FIELD]: new FormControl(EMPTY_STRING),
      }),
      [PostfachFormService.SIGNATUR_FIELD]: new FormControl(EMPTY_STRING),
    });
  }

  protected doSubmit(): Observable<StateResource<PostfachResource | HttpError>> {
    const value: Postfach = this.getFormValue();
    if (this.shouldSkipAbsender(value)) {
      delete value.absender;
    }
    return this.postfachService.save(value);
  }

  private shouldSkipAbsender(postfach: Postfach): boolean {
    return (
      isNil(this.source?.absender) &&
      Object.values(postfach.absender).every((v) => isNil(v) || v.length === 0)
    );
  }

  protected getPathPrefix(): string {
    return 'settingBody';
  }

  public get invalidEmpty(): boolean {
    return this.form.invalid;
  }

  public submitWithProgress(): Observable<boolean> {
    return this.submit().pipe(
      map(
        (stateResource: StateResource<PostfachResource | ProblemDetail>) => stateResource.loading,
      ),
    );
  }
}
