import {
  CommandResource,
  CommandService,
  CreateCommand,
  CreateCommandProps,
} from '@alfa-client/command-shared';
import { EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { hasLink } from '@ngxp/rest';
import { Observable } from 'rxjs';

import * as BescheidSelectors from './bescheid.selectors';

@Injectable({ providedIn: 'root' })
export class BescheidFacade {
  constructor(
    private store: Store,
    private commandService: CommandService,
  ) {}

  public getBescheidCommand(): Observable<StateResource<CommandResource>> {
    return this.store.select(BescheidSelectors.bescheidCommand);
  }

  public createBescheid(
    vorgangWithEingang: VorgangWithEingangResource,
    command: CreateCommand,
  ): void {
    if (hasLink(vorgangWithEingang, VorgangWithEingangLinkRel.CREATE_BESCHEID)) {
      return this.createBescheidKiel(vorgangWithEingang, command);
    }
    if (hasLink(vorgangWithEingang, VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT)) {
      return this.createBescheidDraft(vorgangWithEingang, command);
    }
    throw new Error('Missing Link: CREATE_BESCHEID or CREATE_BESCHEID_DRAFT expected');
  }

  public createBescheidKiel(
    vorgangWithEingang: VorgangWithEingangResource,
    command: CreateCommand,
  ): void {
    const createCommandProps: CreateCommandProps = {
      resource: vorgangWithEingang,
      linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID,
      command,
    };
    this.commandService.createCommandByProps(createCommandProps);
  }

  public createBescheidDraft(
    vorgangWithEingang: VorgangWithEingangResource,
    command: CreateCommand,
  ): void {
    const createCommandProps: CreateCommandProps = {
      resource: vorgangWithEingang,
      linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
      command,
      snackBarMessage: EMPTY_STRING,
    };
    this.commandService.createCommandByProps(createCommandProps);
  }
}
