/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { getUrl, Resource } from '@ngxp/rest';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { decodeUrlFromEmbedding } from '../tech.util';
import { ToResourceUriPipe } from './to-resource-uri.pipe';

describe('ToResourceUriPipe', () => {
  const base64ResultWithSlash: string = '/subjects?_d=1+2';
  const pipe = new ToResourceUriPipe();

  it('create an instance', () => {
    expect(pipe).toBeTruthy();
  });

  describe('pass resource in pipe', () => {
    const dummyResource: Resource = createDummyResource();

    it('output as expected', () => {
      const pipeResult: string = pipe.transform(dummyResource);

      const result: string = decodeUrlFromEmbedding(pipeResult);

      expect(result).toBe(getUrl(dummyResource));
    });
    it('replace "/" with "_" if necessary', () => {
      const pipeResult: string = pipe.transform({
        _links: { self: { href: base64ResultWithSlash } },
      });

      const result: string = decodeUrlFromEmbedding(pipeResult);

      expect(result).toBe(base64ResultWithSlash);
    });
  });

  describe('pass resource and linkrel in pipe', () => {
    const linkRel: string = 'neuerLink';
    const dummyResourceWithLink: Resource = createDummyResource([linkRel]);

    it('output as expected', () => {
      const pipeResult: string = pipe.transform(dummyResourceWithLink, linkRel);

      const result: string = decodeUrlFromEmbedding(pipeResult);

      expect(result).toBe(getUrl(dummyResourceWithLink, linkRel));
    });
  });
});
