import { BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import {
  StateResource,
  containsLoading,
  getSuccessfullyLoaded,
  hasError,
  isLoaded,
  isNotNil,
} from '@alfa-client/tech-shared';
import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { getUrl } from '@ngxp/rest';
import { Subscription, first } from 'rxjs';
import { BescheidenFormService } from '../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-result-attachments',
  templateUrl: './vorgang-detail-bescheiden-result-attachments.component.html',
  styles: [],
})
export class VorgangDetailBescheidenResultAttachmentsComponent implements OnDestroy, OnInit {
  @Input() public deletable: boolean;

  existingAttachments: BinaryFileResource[] = [];
  uploadedAttachments: StateResource<BinaryFileResource>[] = [];

  private attachmentUploadSubscription: Subscription;

  constructor(
    public formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
  ) {}

  ngOnInit(): void {
    this.loadExistingAttachments();
    this.subscribeToAttachmentUpload();
  }

  ngOnDestroy(): void {
    if (isNotNil(this.attachmentUploadSubscription))
      this.attachmentUploadSubscription.unsubscribe();
  }

  loadExistingAttachments() {
    this.bescheidService
      .getAttachments()
      .pipe(first())
      .subscribe((attachments) => {
        this.uploadedAttachments = [];
        this.existingAttachments = attachments;
      });
  }

  subscribeToAttachmentUpload() {
    this.attachmentUploadSubscription = this.formService
      .getAttachmentUpload()
      .subscribe(
        (stateResource: StateResource<BinaryFileResource>) =>
          (this.uploadedAttachments = this.buildUploadedAttachments(stateResource)),
      );
  }

  buildUploadedAttachments(
    uploadStateResource: StateResource<BinaryFileResource>,
  ): StateResource<BinaryFileResource>[] {
    if (isLoaded(uploadStateResource) || hasError(uploadStateResource)) {
      return [...getSuccessfullyLoaded(this.uploadedAttachments), uploadStateResource];
    } else if (uploadStateResource.loading && !containsLoading(this.uploadedAttachments)) {
      return [...getSuccessfullyLoaded(this.uploadedAttachments), uploadStateResource];
    }
    return this.uploadedAttachments;
  }

  deleteFile(file: BinaryFileResource): void {
    this.formService.deleteFile(file);
    this.uploadedAttachments = this.uploadedAttachments.filter(
      (attachment) => getUrl(attachment.resource) !== getUrl(file),
    );
    this.existingAttachments = this.existingAttachments.filter(
      (attachment) => getUrl(attachment) !== getUrl(file),
    );
  }
}
