import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import {
  StateResource,
  createEmptyStateResource,
  doOnValidStateResource,
  isLoaded,
  isNotNil,
} from '@alfa-client/tech-shared';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { getUrl } from '@ngxp/rest';
import { Observable, Subscription, filter, first, of, startWith, switchMap } from 'rxjs';
import { tap } from 'rxjs/operators';
import { BescheidenFormService } from '../../../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-attachment-hochladen',
  templateUrl: './vorgang-detail-bescheiden-attachment-hochladen.component.html',
  styles: [],
})
export class VorgangDetailBescheidenAttachmentHochladenComponent implements OnInit, OnDestroy {
  uploadInProgress$: Observable<StateResource<BinaryFileResource>>;
  fileList: BinaryFileResource[] = [];
  private deleteFileSubscription: Subscription;

  readonly formServiceClass = BescheidenFormService;

  constructor(
    public readonly formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
    private readonly binaryFileService: BinaryFileService,
  ) {
    this.uploadInProgress$ = of(createEmptyStateResource<BinaryFileResource>());
  }

  ngOnInit(): void {
    this.loadExistingAttachments();
    this.subscribeToFileDeletion();
  }

  ngOnDestroy(): void {
    if (isNotNil(this.deleteFileSubscription)) this.deleteFileSubscription.unsubscribe();
  }

  loadExistingAttachments(): void {
    this.bescheidService
      .getAttachments()
      .pipe(first())
      .subscribe((attachments) => (this.fileList = attachments));
  }

  subscribeToFileDeletion(): void {
    this.deleteFileSubscription = this.formService
      .getFileDelete()
      .subscribe((binaryFileResource) => this.deleteFile(binaryFileResource));
  }

  deleteFile(binaryFileResource: BinaryFileResource) {
    this.fileList = this.fileList.filter((file) => getUrl(file) !== getUrl(binaryFileResource));
  }

  getFileLinkList(): string[] {
    return this.fileList.map((fileResource: BinaryFileResource) => getUrl(fileResource));
  }

  public uploadFile(file: File) {
    this.uploadInProgress$ = this.uploadAndGetFile(file).pipe(
      tap((stateResource) => this.formService.uploadAttachment(stateResource)),
      startWith(createEmptyStateResource<BinaryFileResource>(true)),
    );
  }

  uploadAndGetFile(file: File): Observable<StateResource<BinaryFileResource>> {
    return this.bescheidService.getBescheidDraft().pipe(
      filter(isLoaded),
      first(),
      switchMap((bescheidStateResource: StateResource<BescheidResource>) =>
        this.binaryFileService
          .uploadFile(
            bescheidStateResource.resource,
            BescheidLinkRel.UPLOAD_ATTACHMENT,
            file,
            false,
          )
          .pipe(
            tap((stateResource: StateResource<BinaryFileResource>) =>
              doOnValidStateResource(
                stateResource,
                () => (this.fileList = [...this.fileList, stateResource.resource]),
              ),
            ),
          ),
      ),
    );
  }
}
