import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { StateResource } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Observable, tap } from 'rxjs';

@Component({
  selector: 'alfa-vorgang-export-container',
  templateUrl: './vorgang-export-container.component.html',
  styleUrls: ['./vorgang-export-container.component.scss'],
})
export class VorgangExportContainerComponent implements OnInit {
  @Input() vorgangWithEingang: VorgangWithEingangResource;

  @Output() public closeMenu: EventEmitter<void> = new EventEmitter();

  public exportStateResource$: Observable<StateResource<boolean>>;

  constructor(private vorgangService: VorgangService) {}

  ngOnInit(): void {
    this.exportStateResource$ = this.vorgangService
      .getVorgangExport()
      .pipe(tap((vorgangExportStateResource) => this.isExportDone(vorgangExportStateResource)));
  }

  isExportDone(vorgangExportStateResource: StateResource<boolean>): void {
    if (vorgangExportStateResource.resource) this.closeMenu.emit();
  }

  public export(): void {
    this.vorgangService.export(this.vorgangWithEingang);
  }
}
