/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import java.time.ZonedDateTime;
import java.util.Random;
import java.util.UUID;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.test.TestUtils;

public class CommandTestFactory {

	public static final String ID = UUID.randomUUID().toString();
	public static final String VORGANG_ID = VorgangHeaderTestFactory.ID;
	public static final String RELATION_ID = UUID.randomUUID().toString();
	private static final ZonedDateTime CREATED_AT = ZonedDateTime.now();
	public static final CommandStatus STATUS = CommandStatus.FINISHED;
	public static final String ORDER = LoremIpsum.getInstance().getWords(1);

	public static final long RELATION_VERSION = new Random().nextLong();

	public static Command create() {
		return createBuilder().build();
	}

	public static Command.CommandBuilder createBuilder() {
		return Command.builder()
				.id(ID)
				.vorgangId(VORGANG_ID)
				.relationId(RELATION_ID)
				.createdAt(CREATED_AT)
				.status(STATUS)
				.order(ORDER)
				.createdBy(UserProfileTestFactory.ID)
				.createdByName(UserProfileTestFactory.FULLNAME);
	}

	public static CreateCommand createCreateCommand() {
		return createCreateCommandBuilder().build();
	}

	public static CreateCommand.CreateCommandBuilder createCreateCommandBuilder() {
		return CreateCommand.builder()
				.vorgangId(VORGANG_ID)
				.relationId(RELATION_ID)
				.relationVersion(RELATION_VERSION)
				.order(ORDER);
	}

	public static String buildStatusPatchContent(CommandStatus status) {
		return TestUtils.loadTextFile("jsonTemplates/command/statusPatch.json.tmpl", status.name());
	}

	public static String buildCreateVorgangCommandContent(String order) {
		return TestUtils.loadTextFile("jsonTemplates/command/createVorgangCommand.json.tmpl", order);
	}
}