package de.ozgcloud.alfa.loeschanforderung;

import static de.ozgcloud.alfa.common.command.CommandController.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.nio.charset.StandardCharsets;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.errorhandling.ExceptionController;
import de.ozgcloud.alfa.loeschanforderung.LoeschAnforderungController.LoeschAnforderungByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import lombok.SneakyThrows;

class LoeschAnforderungByVorgangControllerTest {

	@InjectMocks
	private LoeschAnforderungByVorgangController controller;

	@Mock
	private LoeschAnforderungService service;

	@Mock
	private VorgangController vorgangController;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new ExceptionController()).build();
	}

	@DisplayName("Create LoeschAnforderung")
	@Nested
	class TestCreateLoeschAnforderung {

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandArgumentCaptor;

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void init() {
			when(vorgangController.getVorgang(any())).thenReturn(vorgang);
			when(service.createLoeschAnforderung(any(), any(), anyLong())).thenReturn(CommandTestFactory.create());
		}

		@SneakyThrows
		@Test
		void shouldLoadVorgang() {
			doRequest();

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldCallService() {
			doRequest();

			verify(service).createLoeschAnforderung(createCommandArgumentCaptor.capture(), eq(vorgang), eq(VorgangHeaderTestFactory.VERSION));
			assertThat(createCommandArgumentCaptor.getValue().getCommandOrder()).isEqualTo(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN);
		}

		@SneakyThrows
		@Test
		void shouldReturnLinkToCommand() {
			doRequest().andExpect(header().stringValues("location", "http://localhost" + COMMANDS_PATH + "/" + CommandTestFactory.ID));
		}

		@SneakyThrows
		private ResultActions doRequest() throws Exception {
			var requestBody = CommandTestFactory.buildCreateVorgangCommandContent(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN.name());
			return mockMvc.perform(post(LoeschAnforderungByVorgangController.BASE_PATH,
					VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.VERSION)
							.content(requestBody).contentType(MediaType.APPLICATION_JSON).characterEncoding(StandardCharsets.UTF_8.name()))
					.andExpect(status().isCreated());
		}
	}

}