package de.ozgcloud.alfa.export;

import static de.ozgcloud.alfa.common.TestUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.file.ExportFileService;
import de.ozgcloud.alfa.historie.ExportHistorieService;
import de.ozgcloud.alfa.kommentar.ExportKommentarService;
import de.ozgcloud.alfa.kommentar.KommentarsExportDataTestFactory;
import de.ozgcloud.alfa.postfach.ExportNachrichtService;
import de.ozgcloud.alfa.postfach.PostfachMail;
import de.ozgcloud.alfa.postfach.PostfachMailExportDataTestFactory;
import de.ozgcloud.alfa.postfach.PostfachMailTestFactory;
import de.ozgcloud.alfa.vorgang.ExportVorgangService;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.common.test.ITCase;

@ITCase
class ExportServiceITCase {

	@SpyBean
	private ExportFileService exportFileService;
	@SpyBean
	private ExportVorgangService exportVorgangService;
	@MockBean
	private ExportHistorieService exportHistorieService;
	@MockBean
	private ExportKommentarService exportKommentarService;
	@MockBean
	private BinaryFileService binaryFileService;
	@MockBean
	private ExportNachrichtService exportNachrichtService;
	@MockBean
	private UserService userService;
	@Autowired
	private ExportService exportService;

	@Nested
	class TestWriteExport {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final PostfachMail postfachMail = PostfachMailTestFactory.createBuilder().id(createMongoDbObjectId()).build();

		@BeforeEach
		void setup() {
			doReturn(vorgang).when(exportVorgangService).getVorgang(VorgangHeaderTestFactory.ID);
			doReturn(Stream.of(OzgFileTestFactory.createWithUniqueId())).when(exportFileService).getRepresentations(vorgang);
			doReturn(Stream.of(OzgFileTestFactory.createWithUniqueId())).when(exportFileService).getAttachments(vorgang);
			doReturn(Stream.of(OzgFileTestFactory.createWithUniqueId())).when(binaryFileService).getFiles(postfachMail.getAttachments());
			doReturn(UserProfileTestFactory.create()).when(userService).getById(UserProfileTestFactory.ID);
			doNothing().when(exportFileService).writeOzgFile(any(), any());
			when(exportHistorieService.createHistorienProtokollInformationTypes(vorgang)).thenReturn(Stream.empty());
			when(exportKommentarService.createExportData(vorgang)).thenReturn(KommentarsExportDataTestFactory.create());
			when(exportNachrichtService.createExportData(vorgang)).thenReturn(PostfachMailExportDataTestFactory.create());
		}

		@Test
		void shouldNotThrowException() {
			assertDoesNotThrow(
					() -> exportService.writeExport(VorgangHeaderTestFactory.ID, UUID.randomUUID().toString(), new ByteArrayOutputStream()));
		}
	}
}
